package info.aspirecn.iov.yysj.product.check.pre;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.netflix.zuul.ZuulFilter;
import com.netflix.zuul.context.RequestContext;
import info.aspirecn.cloud.yysj.commons.lang.BillItemResult;
import info.aspirecn.cloud.yysj.commons.lang.Constants;
import info.aspirecn.cloud.yysj.commons.lang.ProductCheckRequest;
import info.aspirecn.iov.yysj.product.check.common.*;
import info.aspirecn.iov.yysj.product.check.entity.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.apache.tomcat.util.bcel.Const;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.netflix.zuul.filters.support.FilterConstants;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

/**
 * 运单信息核验服务,
 */
@Slf4j
public class WayBillFilter extends ZuulFilter {
	@Autowired
	private ObjectMapper mapper;
	@Autowired
	private ZuulHelper helper;
	@Autowired
	private TransferParam transferParam;
	@Autowired
	private Property property;
	@Override
	public boolean shouldFilter() {
		HttpServletRequest request = RequestContext.getCurrentContext().getRequest();
		request.setAttribute("result", "0");
		request.setAttribute(Constants.price,"0");
		request.setAttribute("isCharge","0");

		String url = request.getRequestURI();

		return Constants.ProductCheckUrl.BILLWAYURL.getValue().equals(url);
	}

	@Override
	public Object run() {


		BillItemResult billItemResult = new BillItemResult();
		String accessId = UUID.randomUUID().toString().replace("-","");
		HttpServletRequest request = RequestContext.getCurrentContext().getRequest();
		try {
			//从请求头中获取token
			String token = request.getHeader(Constants.HEADER_TOKEN_KEY);
			int price =0;
			String yysjTraceId = request.getHeader(Constants.HEADER_TRACEID_KEY);
			String userId = request.getHeader(Constants.USER_ID_KEY);
			log.info("WayBillFilter.userId:{}", userId);

			String body = helper.getCustomBody(request);
			ProductCheckRequest productCheckRequest = mapper.readValue(body, ProductCheckRequest.class);
			List<String> sjjhProductIds = productCheckRequest.getSjjhProductId();
			int sjjhTraceIdIndex = 0;
			String result = "1";
			List<String> emptyList = new ArrayList<String>();

			if(sjjhProductIds.size() ==1){
				SjjhTransferReq sjjhTransferReq = new SjjhTransferReq();
				sjjhTransferReq.setAccessId(accessId);
				sjjhTransferReq.setSjjhProductId(sjjhProductIds.get(0));
				Map<String,Object> customMap =productCheckRequest.getCustomBody();
				/*String yysjCustomStr = mapper.writeValueAsString(productCheckRequest.getCustomBody());
				Map<String,String> customMap = mapper.readValue(yysjCustomStr,Map.class);*/
				//转换成数据交互的请求参数信息
				Object customObject = transferParam.getCustomBody(customMap,productCheckRequest.getSjjhProductId().get(0),emptyList);
				sjjhTransferReq.setCustomBody(customObject);
				//设置云验平台资格校验项校验内容，用于返回结果一致性比较
				sjjhTransferReq.setOriginCustomBody(customMap);
				sjjhTransferReq.setToken(token);
				ResponseObject responseObject = helper.getResult(sjjhTransferReq,billItemResult,productCheckRequest.getProductCode(), userId);
				price = responseObject.getPrice();
			}else{


				SjjhTransferReq sjjhTransferReq = new SjjhTransferReq();
				sjjhTransferReq.setAccessId(accessId);
				String sjjhProductId = property.getProductMap().get("qyyyxkyz_tyshdm");
				sjjhTransferReq.setSjjhProductId(sjjhProductId);
				sjjhTransferReq.setToken(token);
				Map<String,Object> customMap = productCheckRequest.getCustomBody();
				/*String yysjCustomStr = mapper.writeValueAsString(productCheckRequest.getCustomBody());
				Map<String,String> customMap = mapper.readValue(yysjCustomStr,Map.class);*/
				Map<String,String> customObject = transferParam.getCustomBody(customMap,sjjhProductId,emptyList);
				sjjhTransferReq.setCustomBody(customObject);
				sjjhTransferReq.setOriginCustomBody(customMap);
				ResponseObject responseObject = helper.getResult(sjjhTransferReq,billItemResult,productCheckRequest.getProductCode(), userId);
				 result =billItemResult.getResult();
				price = responseObject.getPrice();
				if(Constants.consistent_code.equals(result)){

					sjjhTraceIdIndex++;
					sjjhTransferReq.setAccessId(accessId);
					sjjhProductId = property.getProducts().get("qyyyxkyz_jyxkzh");
					sjjhTransferReq.setSjjhProductId(sjjhProductId);
					sjjhTransferReq.setToken(token);
					sjjhTransferReq.setSjjhTraceId(yysjTraceId + sjjhTraceIdIndex);
					customMap.put("provinceCode",(String)responseObject.getResultBody());
					customObject = transferParam.getCustomBody(customMap,sjjhProductId,emptyList);
					sjjhTransferReq.setCustomBody(customObject);
					sjjhTransferReq.setOriginCustomBody(customMap);
					responseObject = helper.getResult(sjjhTransferReq,billItemResult,productCheckRequest.getProductCode(), userId);
					result =responseObject.getReuslt();
					price = price+responseObject.getPrice();
				}
			}
			RequestContext.getCurrentContext().getRequest().setAttribute(Constants.price,price);
			request.setAttribute("result", billItemResult.getResult());
		} catch (Exception ex) {
			log.error("Exception:{}",ex);
			billItemResult.setResult("0");
			BillItemResult.ErrorResponse errorResponse = new BillItemResult.ErrorResponse();
			errorResponse.setCode(Constants.ErrorCode.Exception.getErrorCode());
			billItemResult.setExceptionInformation(errorResponse);
		}
		try {
			helper.responseHandle(billItemResult, RequestContext.getCurrentContext());
		} catch (JsonProcessingException e) {
			log.error("JsonProcessingException:{}",e);
		}
		log.info("money r:{}",RequestContext.getCurrentContext().getRequest().getAttribute(Constants.price));
		return null;
	}

	@Override
	public String filterType() {
		return FilterConstants.PRE_TYPE;
	}

	@Override
	public int filterOrder() {
		return FilterConstants.SERVLET_30_WRAPPER_FILTER_ORDER + 3;
	}
}
